/****************************************************************************
*
*   Copyright (c) 2007 www.societyofrobots.com
*   (please link back if you use this code!)
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License version 2 as
*   published by the Free Software Foundation.
*
*   Alternatively, this software may be distributed under the terms of BSD
*   license.
*
*	SoR Utilities v2, June 17th, 2007
*
****************************************************************************/

//AVR includes
#include <avr/io.h>		    // include I/O definitions (port names, pin names, etc)
#include <avr/interrupt.h>	// include interrupt support
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <string.h>

//AVRlib includes
#include "global.h"		// include global settings
#include "buffer.h"		// include buffer function library
#include "uart.h"		// include uart function library
#include "rprintf.h"	// include printf function library
#include "timerx8.h"	// include timer function library (timing, PWM, etc)
#include "a2d.h"		// include A/D converter function library

//define port functions; example: PORT_ON( PORTD, 6);
#define PORT_ON( port_letter, number )			port_letter |= (1<<number)
#define PORT_OFF( port_letter, number )			port_letter &= ~(1<<number)
#define PORT_ALL_ON( port_letter, number )		port_letter |= (number)
#define PORT_ALL_OFF( port_letter, number )		port_letter &= ~(number)
#define FLIP_PORT( port_letter, number )		port_letter ^= (1<<number)
#define PORT_IS_ON( port_letter, number )		( port_letter & (1<<number) )
#define PORT_IS_OFF( port_letter, number )		!( port_letter & (1<<number) )

#define XTAL_CPU          8000000L      /* 8Mhz */
#define UART_BAUD_RATE      38400      /* 9600 baud = 9600 */

#define XTAL              8000000L    // Crystal frequency in Hz



//************CONFIGURE PORTS************
//configure ports for input or output - specific to ATmega8
void configure_ports(void)
	{
	DDRC = 0x00;  //configure all C ports for input
	PORTC = 0x00; //make sure pull-up resistors are turned off
	DDRD = 0xFF;  //configure all D ports for output
	DDRB = 0xC7;  //configure B ports 0, 1, 2, 6, 7 for output (google search '0b11000111 to hex')
	}
//***************************************


//*************INITIALIZATIONS***********
void initialize(void)
	{
	uartInit();  // initialize the UART (serial port)
	uartSetBaudRate(19200);// set the baud rate of the UART for our debug/reporting output
	rprintfInit(uartSendByte);// initialize rprintf system

	timerInit(); // initialize the timer system //breaks servos!!!
	
	configure_ports(); // configure which ports are analog, digital, etc.
	a2dInit(); // initialize analog to digital converter (ADC)
	a2dSetPrescaler(ADC_PRESCALE_DIV32); // configure ADC scaling
	a2dSetReference(ADC_REFERENCE_AVCC); // configure ADC reference voltage

	}
//****************************************


//************DELAY FUNCTIONS************
//wait for X amount of cycles (23 cycles is about .992 milliseconds)
//		this was determined by using an oscilloscope
//to calculate: 23/.992*(time in milliseconds) = number of cycles
//or (number of cycles)*.992/23 = time in milliseconds
//ATmega168 - 28400 cycles per second (28.4 cycles per millisecond)
void delay_cycles(unsigned long int cycles)
	{
	while(cycles > 0)
		cycles--;
	}
//***************************************


//*************LED FUNCTIONS*************
//functions to make coding easier for a beginner
//but could cause port mixup confusion for intermediate users
void LED_on(void)
	{
	PORT_OFF(PORTD, 4);//turn LED on
	}
void LED_off(void)
	{
	PORT_ON(PORTD, 4);//turn LED off
	}
//***************************************


//**************ROBOT WHEELS*************
void servo_left(signed long int speed)
	{
	PORT_ON(PORTD, 7);
	delay_cycles(speed);
	PORT_OFF(PORTD, 0);//keep off
	}
void servo_right(signed long int speed)
	{
	PORT_ON(PORTB, 0);
	delay_cycles(speed);		
	PORT_OFF(PORTD, 1);//keep off
	}
void robot_turn_left(void)//45, 45 maximum
	{
	servo_left(39);//dont move
	servo_right(41);//forward
	}
void robot_turn_right(void)//29, 29 maximum
	{
	servo_left(33);//forward
	servo_right(35);//dont move
	}
void robot_go_straight(void)//29, 45 maximum
	{
	servo_left(29);//forward
	servo_right(45);//forward
	}
void robot_go_reverse(void)//29, 45 maximum
	{
	servo_left(45);//back
	servo_right(29);//back
	}
void hold_position(void)//39, 35
	{
	servo_left(39);//39
	servo_right(35);//35
	}
//***************************************


//***********TRIG LOOKUP TABLES**********
//returns a trig sin or cos calculation value multiplied by 100 (to avoid floating point math)
//returns a trig tan calculation value multiplied by 10 (to avoid floating point math)
//only allows for angles between 0 and 360 degrees

//multiplied by 100 so no floating point math
signed int angtable[73]={100,100,98,97,94,91,87,82,77,71,64,57,50,42,34,26,17,9,0,-9,-17,-26,-34,-42,-50,-57,-64,-71,-77,-82,-87,-91,-94,-97,-98,-100,
						 -100,-100,-98,-97,-94,-91,-87,-82,-77,-71,-64,-57,-50,-42,-34,-26,-17,-9,0,9,17,26,34,42,50,57,64,71,77,82,87,91,94,97,98,100,100};

signed int cos_SoR(long signed int degrees)//returns cos*100
	{
	if (degrees >= 0)//positive angles
		return angtable[degrees/5];
	else
		return -angtable[72-(-degrees)/5];
	}

signed int sin_SoR(long signed int degrees)//returns sin*100
	{
	degrees=degrees - 90;//phase shift 90 degrees

	if (degrees >= 0)//positive angles
		return angtable[degrees/5];
	else
		return -angtable[72-(-degrees)/5];
	}

signed int tan_SoR(long signed int degrees)//returns tan * 10
	{
	//tan(x) = sin(x)/cos(x)
	if (degrees == 90 || degrees == -90 || degrees == 270 || degrees == -270)//blows up
		return 0;//what else should I return?!?!?
	return sin_SoR(degrees)/cos_SoR(degrees)*10;
	}

//***************************************
